import argparse
from collections import Counter
from datetime import datetime

from margument.argument import Argument
from margument.arguments import Arguments

from progscheduler.utils.directory import Directory
from progscheduler.utils.log import throw, show
from progscheduler.utils.reflection import get_class_variables, convert_to_dict


class Specific(Arguments):
    are_configs_saved = False

    def __init__(self):
        self.alias = Argument(name='alias',
                              abbreviation_name='-a',
                              full_name='--alias',
                              help_message='A UNIQUE alias for the file to be scheduled. When creating and/or updating any configurations, '
                                           'this alias needs to be present.',
                              metavar="",
                              to_save=True,
                              is_main=True)

        self.path = Argument(name='path',
                             abbreviation_name='-p',
                             full_name='--path',
                             help_message='Absolute path of the file to be scheduled.',
                             metavar="",
                             to_save=True,
                             default='')

        self.days = Argument(name='days',
                             abbreviation_name='-d',
                             full_name='--days',
                             help_message='Days of the week for when the scheduled job will run. Multiple values can be inserted. Following '
                                          'are the available set of values: \'monday\', \'tuesday\', \'wednesday\', \'thursday\', \'friday\', '
                                          '\'saturday\', \'sunday\', \'everyday\', \'weekdays\' and \'weekends\'. The 3 latter should be used '
                                          'individually.',
                             metavar="",
                             to_save=True,
                             default=['monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday', 'sunday'])

        self.time = Argument(name='time',
                             abbreviation_name='-t',
                             full_name='--time',
                             help_message='Insert a specific time to start the file. example: \'08:15\'. default value is: \'at startup\'. If is '
                                          '\'at startup\' then the file will be scheduled to open first thing when the program runs.',
                             metavar="",
                             to_save=True,
                             default='at startup')

        self.time_to_stop = Argument(name='time_to_stop',
                                     abbreviation_name='-ts',
                                     full_name='--time-to-stop',
                                     help_message='If a time is set, the scheduled job will not run, after specified time in a day. If this argument '
                                                  'equals \"off\", this setting will be ignored. Time is expected to be according to 24 hours cycle. '
                                                  'example: -ss 13:30 (meaning the job will not run if time exceeds 13:30), -ss off',
                                     metavar="",
                                     to_save=True,
                                     default='off')

        self.status = Argument(name='status',
                               abbreviation_name='-st',
                               full_name='--status',
                               help_message='This indicates if a scheduled job is active or inactive. Default value is \"on\". example -st on, '
                                            '-st off',
                               metavar="",
                               to_save=True,
                               default='on')

        self.exclude = Argument(name='exclude',
                                abbreviation_name='-ex',
                                full_name='--exclude',
                                help_message='This indicates the specific days for when a specific scheduled job will not run. Can be added multiple '
                                             'dates in the format dd/mm/yyyy. Default value is empty. example: -ex 29/03/2023 25/12/2023. [NOTE]: '
                                             'Any dates inserted will be replacing dates configured before.',
                                metavar="",
                                to_save=True,
                                default=[])

        self.include = Argument(name='include',
                                abbreviation_name='-in',
                                full_name='--include',
                                help_message='This indicates the specific days for when a specific scheduled job should run that it wouldn\'t '
                                             'normally run. Can be added multiple dates in the format dd/mm/yyyy. Default value is empty. example: '
                                             '-in 14/03/2023 23/06/2023. [NOTE]: Any dates inserted will be replacing dates configured before.',
                                metavar="",
                                to_save=True,
                                default=[])

    def set_are_configs_saved(self, are_configs_saved):
        self.are_configs_saved = are_configs_saved

    def add_arguments(self, args_parser):
        args_parser.add_argument(self.path.abbreviation_name, self.path.full_name,
                                 required=not self.are_configs_saved,
                                 help=self.path.help_message,
                                 metavar=self.path.metavar,
                                 default=argparse.SUPPRESS)

        args_parser.add_argument(self.alias.abbreviation_name, self.alias.full_name,
                                 required=not self.are_configs_saved,
                                 help=self.alias.help_message,
                                 metavar=self.alias.metavar,
                                 default=argparse.SUPPRESS)

        args_parser.add_argument(self.days.abbreviation_name, self.days.full_name,
                                 nargs='*',
                                 choices=['monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday', 'sunday', 'everyday', 'weekdays',
                                          'weekends'],
                                 help=self.days.help_message,
                                 metavar=self.days.metavar,
                                 default=argparse.SUPPRESS)

        args_parser.add_argument(self.time.abbreviation_name, self.time.full_name,
                                 type=str,
                                 help=self.time.help_message,
                                 metavar=self.time.metavar,
                                 default=argparse.SUPPRESS)

        args_parser.add_argument(self.time_to_stop.abbreviation_name, self.time_to_stop.full_name,
                                 help=self.time_to_stop.help_message,
                                 metavar=self.time_to_stop.metavar,
                                 default=argparse.SUPPRESS)

        args_parser.add_argument(self.status.abbreviation_name, self.status.full_name,
                                 choices=['on', 'off'],
                                 help=self.status.help_message,
                                 metavar=self.status.metavar,
                                 default=argparse.SUPPRESS)

        args_parser.add_argument(self.exclude.abbreviation_name, self.exclude.full_name,
                                 nargs='*',
                                 help=self.exclude.help_message,
                                 metavar=self.exclude.metavar,
                                 default=argparse.SUPPRESS)

        args_parser.add_argument(self.include.abbreviation_name, self.include.full_name,
                                 nargs='*',
                                 help=self.include.help_message,
                                 metavar=self.include.metavar,
                                 default=argparse.SUPPRESS)

    def process_arguments(self, settings):
        self.__validate_existence_of_alias(settings[0].user_arguments)
        self.__validate_path(settings[0].user_arguments)
        self.__validate_days(settings[0].user_arguments)
        self.__validate_exclude_include_dates(settings[0].settings_from_file, settings[0].user_arguments)

    def __validate_existence_of_alias(self, user_arguments):
        is_configurable = False
        for var in get_class_variables(self):
            if not isinstance(var[1], Argument):
                continue

            if var[1].name in user_arguments:
                is_configurable = True

        if is_configurable and self.alias.name not in user_arguments:
            throw('An alias is needed for specific configurations.')

    def __validate_days(self, user_arguments):
        if self.days.name in user_arguments:
            if user_arguments.days[0] == 'weekdays':
                user_arguments.days = self.__get_specific_days('weekdays')
            elif user_arguments.days[0] == 'weekends':
                user_arguments.days = self.__get_specific_days('weekends')
            elif user_arguments.days[0] == 'everyday':
                user_arguments.days = self.__get_specific_days('everyday')

    @staticmethod
    def __validate_dates(user_list, current_dates):
        for date in user_list:
            user_date = date.split('/')
            try:
                validate_date = datetime(day=int(user_date[0]), month=int(user_date[1]), year=int(user_date[2]))
                if validate_date < datetime.combine(datetime.today().date(), datetime.min.time()):
                    show('\'' + date + '\': is an old date. It will be ignored.', to_exit=True)
                current_dates.append(validate_date)
            except ValueError:
                throw('\'' + date + '\': date not valid.')

    def __validate_exclude_include_dates(self, file, user_arguments):
        include_dates = self.__get_dates_list(file, user_arguments, self.include.name)
        exclude_dates = self.__get_dates_list(file, user_arguments, self.exclude.name)

        duplicate_dates = set(include_dates) & set(exclude_dates)
        if duplicate_dates:
            message = 'Following dates will exist on both exclude and include lists:'
            for dup in duplicate_dates:
                message += '\n\t\t   - ' + dup.strftime('%d/%m/%Y')
            throw(message)

    def __get_dates_list(self, file, user_arguments, name):
        dates = []
        if self.alias.name in user_arguments and user_arguments.alias in file:
            try:
                self.__validate_dates(file[user_arguments.alias][name], dates)
            except KeyError:
                pass

        if name in user_arguments:
            args = convert_to_dict(user_arguments)
            self.__validate_dates(args[name], dates)

        return dates

    def __validate_path(self, user_arguments):
        if self.path.name in user_arguments:
            if not self.__given_argument_path_exists(user_arguments.path):
                throw('\'' + user_arguments.path + '\' path does not exist.')

    def __get_specific_days(self, days_specified):
        days = []

        if days_specified == 'weekdays':
            for i in range(0, 5):
                days.append(self.days.default[i])
            return days

        if days_specified == 'weekends':
            for i in range(5, 7):
                days.append(self.days.default[i])
            return days

        for i in range(0, 7):
            days.append(self.days.default[i])
        return days

    @staticmethod
    def __given_argument_path_exists(path):
        argument_path = Directory(path)
        return argument_path.exists()



import os.path
from os.path import exists


class Directory:
    current_folder = ''
    last_folder_path = ''

    def __init__(self, root):
        self.root = root
        self.get_current_folder()
        self.get_last_directory()

    def get_last_directory(self):
        self.last_folder_path = os.path.dirname(self.root)

    def get_current_folder(self):
        self.current_folder = os.path.basename(os.path.normpath(self.root))

    def create(self, new_folder):
        return os.path.join(self.root, new_folder)

    def create_folder(self, new_folder):
        new_directory = self.create(new_folder)
        if not os.path.isdir(new_directory):
            os.mkdir(new_directory)

        return new_directory

    def search_through(self):
        return os.walk(self.root)

    def remove(self):
        os.remove(self.root)

    def exists(self):
        return exists(self.root)



import os

import argparse

from margument.non_repeatable_settings import NonRepeatableSettings
from margument.options import Options
from margument.repeatable_settings import RepeatableSettings
from margument.settings_processor import SettingsProcessor

from progscheduler.settings.generic_arguments import Generic
from progscheduler.settings.specific_arguments import Specific
from progscheduler.version.progsettings import get_version


class Manager:
    is_to_configure = False

    def __init__(self):
        self.args = argparse.ArgumentParser()
        self.args.add_argument('--version', action='version', version='%(prog)s ' + str(get_version()))

    def configure_arguments(self):
        # manage specific configurations
        specific = Specific()
        local_settings = RepeatableSettings(path=os.path.join(os.path.dirname(os.path.realpath(__file__)), 'local.yaml'),
                                            program_arguments=specific,
                                            options=Options(show_saved=True, save_main_arg_exists=True))
        specific.are_configs_saved = local_settings.exists()

        # manage generic configurations
        global_settings = NonRepeatableSettings(path=os.path.join(os.path.dirname(os.path.realpath(__file__)), 'global.yaml'),
                                                program_arguments=Generic(),
                                                options=Options(show_saved=True, save_different=True))

        settings_processor = SettingsProcessor([local_settings, global_settings], self.args)
        return settings_processor.run()



from datetime import datetime, timedelta

import schedule


class Scheduler:
    __method_to_schedule = None
    __seconds_delay = 0

    everyday = 'everyday'
    monday = 'monday'
    tuesday = 'tuesday'
    wednesday = 'wednesday'
    thursday = 'thursday'
    friday = 'friday'
    saturday = 'saturday'
    sunday = 'sunday'

    __enabled_days_of_week = {
        monday: False,
        tuesday: False,
        wednesday: False,
        thursday: False,
        friday: False,
        saturday: False,
        sunday: False
    }

    def set_method_to_schedule(self, method_to_schedule):
        self.__method_to_schedule = method_to_schedule

    def process(self, list_of_days_to_enable, time_to_schedule):
        self.__enable_days_of_week(list_of_days_to_enable)

        if time_to_schedule == 'at startup':
            time_to_schedule = self.__get_now_date()

        if self.__is_every_day_enable():
            self.__do_every_day(time_to_schedule)
        else:
            self.__do_specific_days(time_to_schedule)

    def run(self, will_exit):
        while True:
            schedule.run_pending()
            if will_exit and self.__all_jobs_done():
                break

    @staticmethod
    def __all_jobs_done():
        for job in schedule.jobs:
            if job.next_run.day == datetime.utcnow().day:
                return False
        return True

    def __is_every_day_enable(self):
        for day in self.__enabled_days_of_week:
            if not self.__enabled_days_of_week[day]:
                return False
        return True

    def __enable_days_of_week(self, list_of_days_to_enable):
        self.__reset_enabled_days_of_week()
        for day in list_of_days_to_enable:
            self.__enabled_days_of_week[day] = True

    def __reset_enabled_days_of_week(self):
        for day in self.__enabled_days_of_week:
            self.__enabled_days_of_week[day] = False

    def __do_every_day(self, time_to_schedule):
        self.__schedule_method(time_to_schedule, self.everyday)

    def __do_specific_days(self, time_to_schedule):
        for day in self.__enabled_days_of_week:
            if self.__enabled_days_of_week[day]:
                self.__schedule_method(time_to_schedule, day)

    def __schedule_method(self, time_to_schedule, day='everyday'):
        scheduler = schedule.every()

        match day:
            case self.everyday:
                scheduler = scheduler.day
            case self.monday:
                scheduler = scheduler.monday
            case self.tuesday:
                scheduler = scheduler.tuesday
            case self.wednesday:
                scheduler = scheduler.wednesday
            case self.thursday:
                scheduler = scheduler.thursday
            case self.friday:
                scheduler = scheduler.friday
            case self.saturday:
                scheduler = scheduler.saturday
            case self.sunday:
                scheduler = scheduler.sunday

        scheduler.at(time_to_schedule).do(self.__method_to_schedule)

    def __get_now_date(self):
        self.__seconds_delay += 1
        now = datetime.now() + timedelta(seconds=self.__seconds_delay)
        return str(now.hour).zfill(2) + ':' + str(now.minute).zfill(2) + ':' + str(now.second).zfill(2)



import inspect


def get_class_variables(class_to_inspect):
    class_variables = []
    for i in inspect.getmembers(class_to_inspect):
        if not i[0].startswith('_') and not inspect.ismethod(i[1]):
            class_variables.append(i)
    return class_variables


def convert_to_dict(obj):
    class_vars = get_class_variables(obj)

    new_dict = {}
    for var in class_vars:
        new_dict[var[0]] = var[1]

    return new_dict



from progscheduler.utils.log import show, throw
from progscheduler.utils.yaml import read_yaml, write_yaml


class Commands:

    @staticmethod
    def delete_config(user_arguments, file_path):
        saved_configs = read_yaml(file_path)

        try:
            saved_configs.pop(user_arguments.delete)
        except KeyError:
            throw('\"' + user_arguments.delete + '\" setting does not exist.')

        if saved_configs:
            write_yaml(file_path, saved_configs)
        else:
            file_path.open('w').close()

        show('The following configuration \n\t\t\t\'' + user_arguments.delete + '\'\n\t\t was deleted.')

    @staticmethod
    def get_configs(file_path):
        list_of_configs = read_yaml(file_path)

        current_configurations_list = 'Scheduled jobs:'

        for alias, values in list_of_configs.items():
            current_configurations_list += '\n\t\t\t' + str(alias) + ':\n\t\t\t\t'
            for name, value in values.items():
                current_configurations_list += str(name) + ': ' + str(value) + '\n\t\t\t\t'

        show(current_configurations_list)

    @staticmethod
    def get_global_configs(file_path):
        list_of_configs = read_yaml(file_path)

        current_configurations_list = 'Global settings:\n\t\t\t\t'
        for name, value in list_of_configs.items():
            current_configurations_list += str(name) + ': ' + str(value) + '\n\t\t\t\t'

        show(current_configurations_list)



import os
from datetime import datetime

from progscheduler.utils.log import show


def open_program(name, root_path):
    if root_path == '':
        show('Following scheduled job does not have a path: ' + name)
        return

    now = datetime.now()
    os.startfile(root_path)
    show('The following file \n\t\t\t\'' + root_path + '\'\n\t\t has started at ' + str(now.hour).zfill(2) + ':' + str(now.minute).zfill(2) + ':' + str(
        now.second).zfill(2) + '.')



import os
import sys
from datetime import datetime

from progscheduler.jobs import open_program
from progscheduler.settings.manager import Manager
from progscheduler.utils.log import show
from progscheduler.scheduler import Scheduler


def main():
    arguments = get_processed_arguments()

    validate_global_settings(arguments)

    run_scheduler(arguments)


def get_processed_arguments():
    manager = Manager()
    return manager.configure_arguments()


def validate_global_settings(arguments):
    if not arguments['Generic'].run.value:
        sys.exit()

    if arguments['Generic'].exit_when_done.value:
        show('Option: \"exit-when-done\" is enabled. This windows will close automatically when all jobs are done.')


def run_scheduler(arguments):
    show('The program will now start running the scheduler.\n\n\t\t\t*NOTE:* While this is running this window should not be closed. If you are '
         'certain that all scheduled jobs are already finished, then it is safe to close this window.')

    scheduler = Scheduler()

    for program in arguments['Specific']:
        do_scheduled_job(scheduler, arguments['Specific'][program])

    scheduler.run(arguments['Generic'].exit_when_done.value)

    if arguments['Generic'].exit_when_done.value:
        os.system('title kill_current_terminal_window')
        os.system(f'taskkill /f /fi "WINDOWTITLE eq kill_current_terminal_window"')


def do_scheduled_job(scheduler, program):
    if not job_will_run(program):
        return

    scheduler.set_method_to_schedule(lambda: open_program(program.alias.value, program.path.value))
    scheduler.process(
        program.days.value,
        program.time.value
    )


def job_will_run(program):
    if not is_scheduled_today(program.days.value, program.alias.value, program.include.value, program.days.value, program.exclude.value):
        return False

    if is_time_to_stop(program.alias.value, program.time_to_stop.value):
        return False

    if not is_job_active(program.alias.value, program.status.value):
        return False

    return True


def is_time_to_stop(program_name, time_to_stop):
    if time_to_stop != 'off'.lower():
        now = datetime.now()
        time = time_to_stop.split(':')
        if now.hour >= int(time[0]) and now.minute > int(time[1]):
            show('Option: \"time-to-stop\" is enabled for \'' + program_name + '\' and it will not run. Defined time: ' +
                 time_to_stop + '. Current time: ' + str(now.hour).zfill(2) + ':' + str(now.minute).zfill(2))
            return True
    return False


def is_scheduled_today(days_to_schedule, program_name, included_days, days, excluded_days):
    if is_included_day(program_name, included_days, days):
        return True

    if is_excluded_day(program_name, excluded_days):
        return False

    # this condition needs to be the last one
    if datetime.now().strftime("%A").lower() not in days_to_schedule:
        return False

    return True


def is_job_active(program_name, status):
    if status.lower() == 'off'.lower():
        show('\"' + program_name + '\" is inactive and it will not run. Status: OFF.')
        return False
    return True


def is_excluded_day(program_name, excluded_days):
    return check_days(program_name, excluded_days, ['not', 'excluded'])


def is_included_day(program_name, included_days, days):
    if check_days(program_name, included_days, ['', 'included']):
        days.append(datetime.now().strftime("%A").lower())
        return True
    return False


def check_days(program_name, days, message):
    for date in days:
        saved_date = date.split('/')
        validate_date = datetime(
            day=int(saved_date[0]),
            month=int(saved_date[1]),
            year=int(saved_date[2])
        )
        if validate_date == datetime.combine(datetime.today().date(), datetime.min.time()):
            show('Today \"' + program_name + '\" will ' + message[0] + ' run. ' + validate_date.strftime('%d/%m/%Y') + ' is an ' + message[1] + ' date.')
            return True
    return False


if __name__ == '__main__':
    main()



import argparse

from margument.argument import Argument
from margument.arguments import Arguments

from progscheduler.settings.commands import Commands


class Generic(Arguments):
    def __init__(self):
        self.schedules = Argument(name='schedules',
                                  abbreviation_name='-lsch',
                                  full_name='--schedules',
                                  help_message='list all saved scheduled jobs. example: -lsch',
                                  metavar="",
                                  command=Commands.get_configs,
                                  default=False)

        self.settings = Argument(name='settings',
                                 abbreviation_name='-ls',
                                 full_name='--settings',
                                 help_message='list all saved global settings. example: -ls',
                                 metavar="",
                                 command=Commands.get_global_configs,
                                 default=False)

        self.exit_when_done = Argument(name='exit_when_done',
                                       abbreviation_name='',
                                       full_name='--exit-when-done',
                                       help_message='will exit the program when all jobs are done for the current day. the configuration will be '
                                                    'saved. example: True: --exit | False: --no-exit',
                                       metavar="",
                                       to_save=True,
                                       default=False)

        self.delete = Argument(name='delete',
                               abbreviation_name='-del',
                               full_name='--delete',
                               help_message='delete a given file alias from the scheduler. example: -d ThisNameRefersToCurrentProgramToSchedule',
                               metavar="",
                               command=Commands.delete_config)

        self.run = Argument(name='run',
                            abbreviation_name='-r',
                            full_name='--run',
                            help_message='if specified will run the scheduler.',
                            metavar="",
                            default=False)

    def add_arguments(self, args_parser):
        args_parser.add_argument(self.schedules.abbreviation_name, self.schedules.full_name,
                                 action='store_true',
                                 help=self.schedules.help_message,
                                 default=argparse.SUPPRESS)

        args_parser.add_argument(self.settings.abbreviation_name, self.settings.full_name,
                                 action='store_true',
                                 help=self.settings.help_message,
                                 default=argparse.SUPPRESS)

        args_parser.add_argument(self.exit_when_done.full_name,
                                 action=argparse.BooleanOptionalAction,
                                 help=self.exit_when_done.help_message,
                                 metavar=self.exit_when_done.metavar,
                                 default=argparse.SUPPRESS)

        args_parser.add_argument(self.delete.abbreviation_name, self.delete.full_name,
                                 help=self.delete.help_message,
                                 metavar=self.delete.metavar,
                                 default=argparse.SUPPRESS)

        args_parser.add_argument(self.run.abbreviation_name, self.run.full_name,
                                 action='store_true',
                                 help=self.run.help_message,
                                 default=argparse.SUPPRESS)

    def process_arguments(self, settings):
        self.schedules.set_command_args(settings[0].file.path)
        self.settings.set_command_args(settings[1].file.path)
        self.delete.set_command_args((settings[0].user_arguments, settings[0].file.path))



import os
import yaml


def get_version():
    path = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'progsettings.yaml')
    with open(path, 'r') as settings_file:
        settings = yaml.safe_load(settings_file)['prog'.upper()]
        return settings['version'.upper()]



from setuptools import setup
import pathlib

from progscheduler.version.progsettings import get_version

here = pathlib.Path(__file__).parent.resolve()
long_description = (here / "README.md").read_text(encoding="utf-8")

version = get_version()

setup(
    name="progscheduler",
    version=version,
    description="A simple automated task to schedule files to open in specific days of the week or every day at startup or at specific time of the "
                "day.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/zaytiri/program-scheduler",
    project_urls={
        'GitHub': 'https://github.com/zaytiri/program-scheduler',
        'Changelog': 'https://github.com/zaytiri/program-scheduler/blob/main/CHANGELOG.md',
    },
    author="zaytiri",
    classifiers=[
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
    ],
    keywords="program, schedule, scheduler, startup, days, open, files, folders",
    package_data={'progscheduler': ['version/progsettings.yaml']},
    packages=["progscheduler", "progscheduler.settings", "progscheduler.utils", "progscheduler.version"],
    python_requires=">=3.10.6",
    install_requires=[
      "PyYAML~=6.0",
      "schedule~=1.1.0"
    ],
    entry_points={
        "console_scripts": [
            "progscheduler=progscheduler:app.main",
        ],
    }
)